﻿#include "async/thread_pool.h"
#include "mirage.h"

using time_type = decltype(std::chrono::high_resolution_clock::now());
std::chrono::duration<double> delta_time = {};
time_type begin_time = {};
time_type last_time = {};
LLGL::RenderSystemPtr renderer = nullptr;

namespace mirage {
    void on_llgl_log(LLGL::Log::ReportType type, const char* text, void* user_data) {
        switch (type) {
        case LLGL::Log::ReportType::Default:
                spdlog::info(text);
                break;
        case LLGL::Log::ReportType::Error:
                spdlog::error(text);
                break;
        }
    }
    std::string to_string(renderer_api api) {
        switch (api) {
            case renderer_api::dx11: return "Direct3D11";
            case renderer_api::dx12: return "Direct3D12";
            case renderer_api::vulkan: return "Vulkan";
            case renderer_api::opengl: return "OpenGL";
            case renderer_api::metal: return "Metal";
        }
        return "Unknown";
    }
    bool init_renderer(const init_info& in_info) {
        RegisterCallback(on_llgl_log);
        LLGL::RenderSystemDescriptor desc{};
        desc.moduleName = to_string(in_info.api);

        spdlog::info("初始化渲染器: {}", desc.moduleName.c_str());
        renderer = LLGL::RenderSystem::Load(desc);
        if (renderer != nullptr) {
            spdlog::info("渲染器加载成功");
            return true;
        }
        spdlog::error("渲染器加载失败");
        return false;
    }

    bool init(const init_info& in_info) {
    	spdlog::info("初始化 mirage");
        if (!init_renderer(in_info)) {
            return false;
        }
        begin_time = std::chrono::high_resolution_clock::now();
        last_time = std::chrono::high_resolution_clock::now();
        return true;
    }

    void destroy() {
        renderer.reset();
    	spdlog::info("mirage 销毁");
    }

    void update() {
        thread_pool::global().process_main_thread_callbacks();

    	// 更新时间
        const auto& current_time = std::chrono::high_resolution_clock::now();
        delta_time               = current_time - last_time;
        last_time                = current_time;

        std::this_thread::yield();
    }

    bool should_exit() {
        return false;
    }

    int run(const init_info& in_init_info) {
        try {
            if (!init(in_init_info)) {
                spdlog::error("初始化 mirage 失败");
                return -1;
            }
            while (!should_exit()) {
                update();
            }
            destroy();
        } catch (const std::exception& e) {
            spdlog::error("运行时异常: {}", e.what());
            return -1;
        } catch (...) {
            spdlog::error("未知异常");
            return -1;
        }
        return 0;
    }

	const std::chrono::duration<double>& get_delta_time() { return delta_time; }

	std::chrono::duration<double> get_total_time() {
		return std::chrono::high_resolution_clock::now() - begin_time;
	}
}
